package menhirGLR

  1. Overview
  2. Docs

Module MenhirGLRSource

Sourcemodule GLR : sig ... end
Sourcemodule GLRAPI : sig ... end
Sourcemodule GSS : sig ... end
Sourcemodule Input : sig ... end

This module provides an abstraction of the input stream that is convenient for use in an LR or GLR parser. It manages a one-place buffer which stores the lookahead symbol. It also manages two vectors of positions, which are used to translate dates into positions.

Sourcemodule LinearizedArray : sig ... end
Sourcemodule MiniBabySet : sig ... end

This module offers a stripped-down weight-balanced binary tree library.

Sourcemodule MiniVector : sig ... end

This module is a stripped-down version of Base.Vector. It contains just the functionality that is needed by PriorityQueue, which itself is used by GLR.

Sourcemodule PackedIntArray : sig ... end

This module allows packing an array of (small) integers inside a string, using less than one word of memory per array element.

Sourcemodule Path : sig ... end
Sourcemodule PriorityQueue : sig ... end
Sourcemodule RowDisplacementDecode : sig ... end

Row displacement aims to compress a two-dimensional table where some values are considered insignificant.

Sourcemodule StaticVersion : sig ... end
Sourcemodule TableFormat : sig ... end

To find out how the values discussed below are encoded at the bit level, please see TableInterpreter and GLRBackend.

Sourcemodule TableInterpreter : sig ... end

This module decodes the tables generated by the GLR back-end and provides an API that is suitable for use by the GLR engine.

Sourcemodule Tops : sig ... end

This module implements the set of top nodes of the GSS.