Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
MenhirGLRSourceThis module provides an abstraction of the input stream that is convenient for use in an LR or GLR parser. It manages a one-place buffer which stores the lookahead symbol. It also manages two vectors of positions, which are used to translate dates into positions.
This module offers a stripped-down weight-balanced binary tree library.
This module is a stripped-down version of Base.Vector. It contains just the functionality that is needed by PriorityQueue, which itself is used by GLR.
This module allows packing an array of (small) integers inside a string, using less than one word of memory per array element.
Row displacement aims to compress a two-dimensional table where some values are considered insignificant.
To find out how the values discussed below are encoded at the bit level, please see TableInterpreter and GLRBackend.
This module decodes the tables generated by the GLR back-end and provides an API that is suitable for use by the GLR engine.